﻿using Modèle;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace Persistance
{
    public class DataContractPersistance : IPersistanceManager
    {

        // DataContractSerializer pour chaque type
        public DataContractSerializer SerializerReq { get; set; }
            = new DataContractSerializer(typeof(List<Requin>));
        public DataContractSerializer SerializerUsers { get; set; }
            = new DataContractSerializer(typeof(List<Utilisateur>));
        public DataContractSerializer SerializerExperts { get; set; }
            = new DataContractSerializer(typeof(List<Expert>));



        // Chemin des fichiers
        string FilePath { get; set; } = Path.Combine(Directory.GetCurrentDirectory(), "..//..//..//Persistance");



        // Noms des fichiers
        string FileNameReq { get; set; }
            = "ficReq";
        string FileNameReqAdd { get; set; }
            = "ficReqAdd";
        string FileNameReqEdit { get; set; }
            = "ficReqEdit";
        string FileNameUsers { get; set; }
            = "ficUsers";
        string FileNameExperts { get; set; }
            = "ficExperts";



        // Fichiers complets
        string FileReq => Path.Combine(FilePath, FileNameReq);
        string FileReqAdd => Path.Combine(FilePath, FileNameReqAdd);
        string FileReqEdit => Path.Combine(FilePath, FileNameReqEdit);
        string FileUsers => Path.Combine(FilePath, FileNameUsers);
        string FileExperts => Path.Combine(FilePath, FileNameExperts);



        // Listes des requins et utilisateurs
        List<Requin> requins = new List<Requin>();
        List<Requin> requinsAdd = new List<Requin>();
        List<Requin> requinsEdit = new List<Requin>();
        List<Utilisateur> users = new List<Utilisateur>();
        List<Expert> experts = new List<Expert>();



        // Méthodes pour les requins de l'application
        public List<Requin> LoadRequins()
        {
            if (!File.Exists(FileReq))
            {
                throw new FileNotFoundException("The persisance ficReq does not exist");
            }

            using (Stream stream = File.OpenRead(FileReq))
            {
                requins = SerializerReq.ReadObject(stream) as List<Requin>;
            }

            return requins;
        }
        public void SaveRequins(List<Requin> reqs)
        {
            if (!Directory.Exists(FilePath))
            {
                Directory.CreateDirectory(FilePath);
            }

            using (Stream stream = File.Create(FileReq))
            {
                SerializerReq.WriteObject(stream, reqs);
            }
        }



        // Méthodes pour les requins ajoutés par les Utilisateurs
        public List<Requin> LoadRequinsAdd()
        {
            if (!File.Exists(FileReqAdd))
            {
                throw new FileNotFoundException("The persisance file ficReqAdd does not exist");
            }

            using (Stream stream = File.OpenRead(FileReqAdd))
            {
                requinsAdd = SerializerReq.ReadObject(stream) as List<Requin>;
            }

            return requinsAdd;
        }

        public void SaveRequinsAdd(List<Requin> reqs)
        {
            if (!Directory.Exists(FilePath))
            {
                Directory.CreateDirectory(FilePath);
            }

            using (Stream stream = File.Create(FileReqAdd))
            {
                SerializerReq.WriteObject(stream, reqs);
            }
        }



        // Méthodes pour les requins édités par les utilisateurs
        public List<Requin> LoadRequinsEdit()
        {
            if (!File.Exists(FileReqEdit))
            {
                throw new FileNotFoundException("The persisance file ficReqEdit does not exist");
            }

            using (Stream stream = File.OpenRead(FileReqEdit))
            {
                requinsEdit = SerializerReq.ReadObject(stream) as List<Requin>;
            }

            return requinsEdit;
        }

        public void SaveRequinsEdit(List<Requin> reqs)
        {
            if (!Directory.Exists(FilePath))
            {
                Directory.CreateDirectory(FilePath);
            }

            using (Stream stream = File.Create(FileReqEdit))
            {
                SerializerReq.WriteObject(stream, reqs);
            }
        }



        // Méthodes pour les utilisateurs (Utilisateur et Manager)
        public List<Utilisateur> LoadUsers()
        {
            if (!File.Exists(FileUsers))
            {
                throw new FileNotFoundException("The persisance file ficUsers does not exist");
            }

            using (Stream stream = File.OpenRead(FileUsers))
            {
                users = SerializerUsers.ReadObject(stream) as List<Utilisateur>;
            }

            return users;
        }

        public List<Expert> LoadExperts()
        {
            if (!File.Exists(FileExperts))
            {
                throw new FileNotFoundException("The persisance file ficUsers does not exist");
            }

            using (Stream stream = File.OpenRead(FileExperts))
            {
                experts = SerializerExperts.ReadObject(stream) as List<Expert>;
            }

            return experts;
        }

        public void SaveUsers(List<Utilisateur> users)
        {
            if (!Directory.Exists(FilePath))
            {
                Directory.CreateDirectory(FilePath);
            }

            using (Stream stream = File.Create(FileUsers))
            {
                SerializerUsers.WriteObject(stream, users);
            }
        }

        public void SaveExperts(List<Expert> experts)
        {
            if (!Directory.Exists(FilePath))
            {
                Directory.CreateDirectory(FilePath);
            }

            using (Stream stream = File.Create(FileExperts))
            {
                SerializerExperts.WriteObject(stream, experts);
            }
        }

    }
}
